/***********************************************************************
 * File: conf_file_loader-test.c
 * 	(Extracted from
 *	   "$Id: conf_file_loader-test.c,v 1.1 2008/08/09 19:19:06 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  	This C source implements a test program for "load configuration
 *  files" API-s.
 *
 * Copyright (C) 2005-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include "conf_file_loader.h" /* definitions for the API-s */


/* print the Configuration_t structure */
static void print_conf(Configuration_t *conf)
{
	size_t	i, j;

	printf("conf (@%p)\n"
			"\t top_addr = %p.\n"
			"\t file_alloc_sz = %lu.\n"
			"\t rows_n   = %lu.\n"
			"\t rows_alloc_sz   = %lu.\n"
			,	(void *)conf
			,	conf->top_addr
			,		(unsigned long)conf->file_alloc_sz
			,		(unsigned long)conf->rows_n
			,		(unsigned long)conf->rows_alloc_sz
			); (void)fflush(stdout);
	for (i = 0; i < conf->rows_n; ++i) {
		printf("\t conf->rows[%lu] (@%p)\n"
				"\t\t rows[%lu]->key    = @%p \"%s\".\n"
				"\t\t rows[%lu]->vals_n = %lu.\n"
				"\t\t rows[%lu]->vals_alloc_sz = %lu.\n"
				,	(unsigned long)i, (void *)(conf->rows + i)
				,	(unsigned long)i
				,			conf->rows[i].key, conf->rows[i].key
				,	(unsigned long)i
				,			(unsigned long)conf->rows[i].vals_n
				,	(unsigned long)i
				,			(unsigned long)conf->rows[i].vals_alloc_sz
				); (void)fflush(stdout);
		for (j = 0; j < conf->rows[i].vals_n; ++j) {
			printf("\t\t\t rows->vals[%lu] = @%p \"%s\".\n"
					, (unsigned long)j
					,		conf->rows[i].vals[j], conf->rows[i].vals[j]
					); (void)fflush(stdout);
		}
	}

	return;
}


/* Test program entry */
int main(int ac, char **av)
{
	char		*comments = NULL;
	char		val_delimit = '\t';
	Configuration_t	*conf;

	if ((ac == 1)
	|| (comments = (char *)((strstr(av[1], ".properties") != NULL)?
						JAVA_PROPERTY_FILE_COMMENT_CHARS
					: ((strstr(av[1], ".INI") != NULL)
					  || strstr(av[1], ".ini") != NULL)?
						DOS_INI_FILE_COMMENT_CHARS
					: NULL
					)) == NULL) {
		fprintf(stderr, "Usage: %s <.INI or .properties file>\n", av[0]
				); (void)fflush(stderr);
		exit(1);
	}

	printf("<<< load_conf_file() tests...\n"); (void)fflush(stdout);

	val_delimit = (comments == JAVA_PROPERTY_FILE_COMMENT_CHARS)?
							';' : '\t';

	if ((conf = load_conf_file(av[1], comments, val_delimit))
							== NULL) {
		printf("ERROR: load_conf_file(\"%s\", \"%s\", 0x%x) "
				"returns %p (errno = %d).\n"
				,		av[1], comments, (unsigned int)val_delimit
				,	(void *)conf, errno
				); (void)fflush(stdout);
		exit(2);
	}

	print_conf(conf);
	if (conf->rows_n != 0) {
		printf("\n"); (void)fflush(stdout);

		printf("<<< get_row_matched_key() test...\n"
			); (void)fflush(stdout);
		{
			size_t		i;
			char		*key;
			Conf_row_t	*row;

			for (i = 0; i < conf->rows_n; ++i) {
				if ((row = get_row_matched_key(conf, conf->rows[i].key))
															== NULL) {
					printf("!!ERROR!! get_row_matched_key(%p, "
							"conf->rows[%lu]. key = \"%s\") returns %p "
							"(errno = %d).\n"
							, (void *)conf, (unsigned long)i
							, conf->rows[i].key, (void *)row
							, errno
							); (void)fflush(stdout);
				}
				else {
					size_t	j, val_idx = 0;
					char	*value;

					printf("SUCCESS: row->vals_n = %lu. row->key is "
							"\"%s\" @%p. conf->rows[%ld].key is"
							" \"%s\" @%p.\n"
							, (unsigned long)row->vals_n
							, row->key, row->key, (long)i
							, conf->rows[i].key, conf->rows[i].key
							); (void)fflush(stdout);

					printf("\t<<< find_value_matched_key() tests...\n"
							); (void)fflush(stdout);
					for (j = 0, key = row->key; j < row->vals_n; ++j) {
						if ((value = find_value_matched_key(conf
													, key
													, row->vals[j]
													, &val_idx
													)) == NULL) {
							printf("\t!!ERROR!! "
									"find_value_matched_key("
									"%p, \"%s\", row->vals[%ld] = "
									"\"%s\", %p) returns "
									"%p (errno = %d).\n"
									, (void *)conf, key, (long)j
									, row->vals[j], (void *)&val_idx
									, value, errno
									); (void)fflush(stdout);
						}
						else {
							Conf_row_t	*r = row;
							for (;;) {
									printf("\tSUCCESS: val_idx = %ld. "
											"value is \"%s\" @%p. "
											"row @%p. "
											"row->vals[%ld] is "
											"\"%s\" @%p.\n"
											, (long)val_idx
											, value, value
											, (void *)r
											, (long)j
											, r->vals[j], r->vals[j]
											); (void)fflush(stdout);

								if (((r + 1
										>= conf->rows + conf->rows_n)
								|| (r + 1)->vals_n < val_idx)
								|| strcmp((r + 1)->key, key) != 0) {
									break;
								}
								++r;
							}
						}
					}
				}
				printf("\n"); (void)fflush(stdout);
			}
		}

	}

	printf("<<< destroy_conf() tests...\n"); (void)fflush(stdout);
	destroy_conf(conf);
	print_conf(conf);
	conf	= NULL;
	printf("\n"); (void)fflush(stdout);

	printf(">>> All tests finished!\n"); (void)fflush(stdout);

	exit(0);
}
